package com.yida.data.school.smart.excel;

import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.alibaba.fastjson.JSON;
import com.yida.data.common.core.entity.CurrentUser;
import com.yida.data.school.dto.smart.GuideRosterErrorExportDTO;
import com.yida.data.school.dto.smart.GuideRosterImportDTO;
import com.yida.data.school.smart.service.ImportGuideRosterService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.ehcache.impl.internal.concurrent.ConcurrentHashMap;

import java.util.List;

/**
 * Excel导入指南花名册信息监听类
 *
 * @author ZYJ
 * @date 2023/6/1
 */
@Slf4j
@RequiredArgsConstructor
public class GuideRosterExcelListener extends AnalysisEventListener<GuideRosterImportDTO> {

    private final ImportGuideRosterService importGuideRosterService;

    /**
     * 错误信息集合
     */
    private final List<GuideRosterErrorExportDTO> errorList;

    /**
     * 指南id
     */
    private final Long guideId;

    /**
     * 当前导入数据唯一标识
     */
    private final String uuid;

    /**
     * 当前登录用户信息
     */
    private final CurrentUser currentUser;

    /**
     * 每次存储条数
     * 每隔5条存储数据库, 实际使用中可以3000条, 然后清理list, 方便内存回收
     */
    private static final int BATCH_COUNT = 3000;

    /**
     * 需要保存的数据集合
     */
    ConcurrentHashMap<Integer, GuideRosterImportDTO> concurrentHashMap = new ConcurrentHashMap<>();

    /**
     * excel总数量
     */
    private static Integer totalCount = 0;

    /**
     * 处理次数
     */
    private static Integer handleNumber = 0;

    /**
     * 解析成功一条数据调用一次
     *
     * @param dto             Excel指南花名册信息导入类
     * @param analysisContext analysisContext类型
     * @author ZYJ
     * @date 2021/8/23 16:12
     */
    @Override
    public void invoke(GuideRosterImportDTO dto, AnalysisContext analysisContext) {
        totalCount = analysisContext.readSheetHolder().getApproximateTotalRowNumber() - 1;
        // 获取读取的第几条数据
        Integer i = analysisContext.readRowHolder().getRowIndex();
        log.info("解析到第: {}条数据: {}", i, JSON.toJSONString(dto));
        concurrentHashMap.put(i, dto);
        // 达到BATCH_COUNT了, 需要去存储一次数据库, 防止数据几万条数据在内存, 容易OOM
        if (concurrentHashMap.size() >= BATCH_COUNT) {
            handleNumber += 1;
            saveData();
            // 存储完成清理集合
            concurrentHashMap.clear();
        }
    }

    /**
     * 所有数据解析完成调用
     *
     * @param analysisContext analysisContext类
     * @author ZYJ
     * @date 2021/8/23 16:18
     */
    @Override
    public void doAfterAllAnalysed(AnalysisContext analysisContext) {
        // 这里也要保存数据, 确保最后遗留的数据也存储到数据库
        saveData();
        log.info("所有数据解析完成！");
        if (errorList.size() > 0) {
            log.error("错误数据有: {}条", errorList.size());
        }
    }

    /**
     * 保存导入数据
     *
     * @author ZYJ
     * @date 2021/8/23 16:20
     */
    private void saveData() {
        log.info("{}条数据，开始存储数据库！", concurrentHashMap.size());
        importGuideRosterService.saveGuideRosterExcelData(concurrentHashMap, errorList, guideId, uuid,
                currentUser, totalCount, handleNumber * BATCH_COUNT);
        log.info("存储数据库成功！");
    }
}
