package com.yida.data.user.excel;

import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.alibaba.fastjson.JSON;
import com.yida.data.user.dto.StaffImportDTO;
import com.yida.data.user.dto.WelcomeStudentImportDTO;
import com.yida.data.user.service.EduStaffService;
import com.yida.data.user.service.ImportWelcomeStudentService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.ehcache.impl.internal.concurrent.ConcurrentHashMap;

/**
 * Excel 职工信息监听类
 */
@Slf4j
@RequiredArgsConstructor
public class StaffDataExcelListener extends AnalysisEventListener<StaffImportDTO> {

    private final EduStaffService eduStaffService;

    /**
     * 错误信息集合
     */
    private final ConcurrentHashMap<Integer, Object> errorMap;

    /**
     * 学校id
     */
    private final Long schoolId;

    /**
     * 时间戳
     */
    private final long timestamp;

    /**
     * 每次存储条数 每隔5条存储数据库, 实际使用中可以3000条, 然后清理list, 方便内存回收
     */
    private static final int BATCH_COUNT = 3000;

    /**
     * 需要保存的数据集合
     */
    ConcurrentHashMap<Integer, StaffImportDTO> concurrentHashMap = new ConcurrentHashMap<>();

    /**
     * 解析成功一条数据调用一次
     */
    @Override
    public void invoke(StaffImportDTO staffImportDTO, AnalysisContext analysisContext) {
        // 获取读取的第几条数据
        Integer i = analysisContext.readRowHolder().getRowIndex();
        log.info("解析到第: {}条数据: {}", i, JSON.toJSONString(staffImportDTO));
        concurrentHashMap.put(i, staffImportDTO);
        // 达到BATCH_COUNT了, 需要去存储一次数据库, 防止数据几万条数据在内存, 容易OOM
        if (concurrentHashMap.size() >= BATCH_COUNT) {
            saveData();
            // 存储完成清理集合
            concurrentHashMap.clear();
        }
    }

    /**
     * 所有数据解析完成调用
     *
     * @param analysisContext analysisContext类
     * @author ZYJ
     * @date 2021/8/23 16:18
     */
    @Override
    public void doAfterAllAnalysed(AnalysisContext analysisContext) {
        // 这里也要保存数据, 确保最后遗留的数据也存储到数据库
        saveData();
        log.info("所有数据解析完成！");
        if (errorMap.size() > 0) {
            log.error("错误数据有: {}条", errorMap.size());
        }
    }

    /**
     * 保存导入数据
     */
    private void saveData() {
        log.info("{}条数据，开始存储数据库！", concurrentHashMap.size());
        eduStaffService.saveExcelData(concurrentHashMap, errorMap, schoolId, timestamp);
        log.info("存储数据库成功！");
    }
}
